Class: RSpec::Queue::QueueReporter

Inherits:
SimpleDelegator
  • Object
show all
Defined in:
lib/rspec/queue.rb

Instance Method Summary collapse

Constructor Details

#initialize(reporter, queue, example) ⇒ QueueReporter

Returns a new instance of QueueReporter.



414
415
416
417
418
# File 'lib/rspec/queue.rb', line 414

def initialize(reporter, queue, example)
  @queue = queue
  @example = example
  super(reporter)
end

Instance Method Details

#acknowledgeObject



443
444
445
# File 'lib/rspec/queue.rb', line 443

def acknowledge
  @queue.acknowledge(@example.queue_entry)
end

#cancel_run!Object



436
437
438
439
440
441
# File 'lib/rspec/queue.rb', line 436

def cancel_run!
  # Remove the requeued example from the list of examples ran
  # Otherwise some formatters might break because the example state is reset
  examples.pop
  nil
end

#report_failure!Object



424
425
426
# File 'lib/rspec/queue.rb', line 424

def report_failure!
  @queue.report_failure!
end

#report_requeue!Object



428
429
430
# File 'lib/rspec/queue.rb', line 428

def report_requeue!
  @queue.report_requeue!
end

#report_success!Object



420
421
422
# File 'lib/rspec/queue.rb', line 420

def report_success!
  @queue.report_success!
end

#requeueObject



432
433
434
# File 'lib/rspec/queue.rb', line 432

def requeue
  @queue.requeue(@example.queue_entry)
end